@@ -14,7 +14,7 @@ prop_commutativeAdd n m = n + m == m + n
 
 and testing:
 
->>> quickcheck prop_commutativeAdd
+>>> quickCheck prop_commutativeAdd
 +++ OK, passed 100 tests.
 
 which tests @prop_commutativeAdd@ on 100 random @(Integer, Integer)@ pairs.
@@ -146,7 +146,6 @@ module Test.QuickCheck
   , shrinkList
   , shrinkIntegral
   , shrinkRealFrac
-  , shrinkRealFracToInteger
     -- ** Helper functions for implementing coarbitrary
   , variant
   , coarbitraryIntegral
@@ -184,6 +183,7 @@ module Test.QuickCheck
     -- *** Controlling property execution
   , verbose
   , once
+  , again
   , within
   , noShrinking
     -- *** Conjunction and disjunction
