@@ -67,26 +67,12 @@ infixr 1 .||.
 -- * Property and Testable types
 
 -- | The type of properties.
---
--- Backwards combatibility note: in older versions of QuickCheck
--- 'Property' was a type synonym for @'Gen' 'Prop'@, so you could mix
--- and match property combinators and 'Gen' monad operations. Code
--- that does this will no longer typecheck.
--- However, it is easy to fix: because of the 'Testable' typeclass, any
--- combinator that expects a 'Property' will also accept a @'Gen' 'Property'@.
--- If you have a 'Property' where you need a @'Gen' 'a'@, simply wrap
--- the property combinator inside a 'return' to get a @'Gen' 'Property'@, and
--- all should be well.
 newtype Property = MkProperty { unProperty :: Gen Prop }
 
 -- | The class of things which can be tested, i.e. turned into a property.
 class Testable prop where
   -- | Convert the thing to a property.
   property :: prop -> Property
-  -- | If true, the property will only be tested once.
-  -- However, if used inside a quantifier, it will be tested normally.
-  exhaustive :: prop -> Bool
-  exhaustive _ = False
 
 -- | If a property returns 'Discard', the current test case is discarded,
 -- the same as if a precondition was false.
@@ -94,22 +80,18 @@ data Discard = Discard
 
 instance Testable Discard where
   property _ = property rejected
-  exhaustive _ = True
 
 instance Testable Bool where
   property = property . liftBool
-  exhaustive _ = True
 
 instance Testable Result where
   property = MkProperty . return . MkProp . protectResults . return
-  exhaustive _ = True
 
 instance Testable Prop where
   property (MkProp r) = MkProperty . return . MkProp . ioRose . return $ r
-  exhaustive _ = True
 
 instance Testable prop => Testable (Gen prop) where
-  property mp = MkProperty $ do p <- mp; unProperty (property p)
+  property mp = MkProperty $ do p <- mp; unProperty (again p)
 
 instance Testable Property where
   property = property . unProperty
@@ -125,6 +107,11 @@ morallyDubiousIOProperty = ioProperty -- Silly names aren't all they're cracked
 --
 -- For more advanced monadic testing you may want to look at
 -- "Test.QuickCheck.Monadic".
+--
+-- Note that if you use 'ioProperty' on a property of type @IO Bool@,
+-- or more generally a property that does no quantification, the property
+-- will only be executed once. To test the property repeatedly you must
+-- use the 'again' combinator.
 ioProperty :: Testable prop => IO prop -> Property
 ioProperty = MkProperty . fmap (MkProp . ioRose . fmap unProp) . promote . fmap (unProperty . property)
 
@@ -243,7 +230,7 @@ succeeded, failed, rejected :: Result
       , expect       = True
       , reason       = ""
       , theException = Nothing
-      , abort        = False
+      , abort        = True
       , labels       = Map.empty
       , stamp        = Set.empty
       , callbacks    = []
@@ -348,6 +335,10 @@ expectFailure = mapTotalResult (\res -> res{ expect = False })
 once :: Testable prop => prop -> Property
 once = mapTotalResult (\res -> res{ abort = True })
 
+-- | Undoes the effect of 'once'.
+again :: Testable prop => prop -> Property
+again = mapTotalResult (\res -> res{ abort = False })
+
 -- | Attaches a label to a property. This is used for reporting
 -- test case distribution.
 label :: Testable prop => String -> prop -> Property
@@ -412,15 +403,13 @@ within n = mapRoseResult f
 -- test case generator.
 forAll :: (Show a, Testable prop)
        => Gen a -> (a -> prop) -> Property
-forAll gen pf =
-  MkProperty $
-  gen >>= \x ->
-    unProperty (counterexample (show x) (pf x))
+forAll gen pf = forAllShrink gen (\_ -> []) pf
 
 -- | Like 'forAll', but tries to shrink the argument for failing test cases.
 forAllShrink :: (Show a, Testable prop)
              => Gen a -> (a -> [a]) -> (a -> prop) -> Property
 forAllShrink gen shrinker pf =
+  again $
   MkProperty $
   gen >>= \x ->
     unProperty $
@@ -432,6 +421,7 @@ forAllShrink gen shrinker pf =
 -- makes 100 random choices.
 (.&.) :: (Testable prop1, Testable prop2) => prop1 -> prop2 -> Property
 p1 .&. p2 =
+  again $
   MkProperty $
   arbitrary >>= \b ->
     unProperty $
@@ -445,6 +435,7 @@ p1 .&&. p2 = conjoin [property p1, property p2]
 -- | Take the conjunction of several properties.
 conjoin :: Testable prop => [prop] -> Property
 conjoin ps =
+  again $
   MkProperty $
   do roses <- mapM (fmap unProp . unProperty . property) ps
      return (MkProp (conj id roses))
@@ -481,6 +472,7 @@ p1 .||. p2 = disjoin [property p1, property p2]
 -- | Take the disjunction of several properties.
 disjoin :: Testable prop => [prop] -> Property
 disjoin ps =
+  again $
   MkProperty $
   do roses <- mapM (fmap unProp . unProperty . property) ps
      return (MkProp (foldr disj (MkRose failed []) roses))
